"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const jsonfile_1 = require("jsonfile");
const path_1 = require("path");
const util_1 = require("util");
const config_1 = require("../config");
const command_constants_1 = require("../constants/command.constants");
const messages_constants_1 = require("../constants/messages.constants");
const data_1 = require("../libs/data");
const process_manager_1 = require("../libs/process-manager");
const utils_1 = require("../libs/utils");
class Start extends command_1.Command {
    async run() {
        let { flags: userFlags } = this.parse(Start);
        let environmentInfo;
        // We are in a container, env file is ready and relative to the Dockerfile
        if (userFlags.container) {
            const environment = await jsonfile_1.readFile(userFlags.data, 'utf-8');
            environmentInfo = {
                dataFile: userFlags.data,
                name: environment.name,
                protocol: environment.https ? 'https' : 'http',
                port: environment.port
            };
        }
        else {
            const environments = await data_1.parseDataFile(userFlags.data);
            userFlags = await utils_1.promptEnvironmentChoice(userFlags, environments);
            try {
                environmentInfo = await data_1.prepareData(environments, {
                    index: userFlags.index,
                    name: userFlags.name,
                    port: userFlags.port,
                    pname: userFlags.pname
                });
            }
            catch (error) {
                this.error(error.message);
            }
            if (!utils_1.portIsValid(environmentInfo.port)) {
                this.error(util_1.format(messages_constants_1.Messages.CLI.PORT_IS_NOT_VALID, environmentInfo.port));
            }
            if (await utils_1.portInUse(environmentInfo.port)) {
                this.error(util_1.format(messages_constants_1.Messages.CLI.PORT_ALREADY_USED, environmentInfo.port));
            }
        }
        try {
            const runningProcesses = await process_manager_1.ProcessManager.list();
            if (runningProcesses
                .map((process) => process.name)
                .includes(environmentInfo.name)) {
                this.error(util_1.format(messages_constants_1.Messages.CLI.PROCESS_NAME_USED_ERROR, environmentInfo.name));
            }
            const process = await process_manager_1.ProcessManager.start({
                max_restarts: 1,
                wait_ready: true,
                min_uptime: 10000,
                kill_timeout: 2000,
                args: ['--data', environmentInfo.dataFile],
                error: path_1.join(config_1.Config.logsPath, `${environmentInfo.name}-error.log`),
                output: path_1.join(config_1.Config.logsPath, `${environmentInfo.name}-out.log`),
                name: environmentInfo.name,
                script: path_1.resolve(__dirname, '../libs/server.js'),
                exec_mode: 'fork'
            });
            if (process[0].pm2_env.status === 'errored') {
                // if process is errored we want to delete it
                await process_manager_1.ProcessManager.delete(environmentInfo.name);
                this.error(util_1.format(messages_constants_1.Messages.CLI.PROCESS_START_LOG_ERROR, environmentInfo.name, environmentInfo.name));
            }
            this.log(messages_constants_1.Messages.CLI.PROCESS_STARTED, environmentInfo.protocol, environmentInfo.port, process[0].pm2_env.pm_id, process[0].pm2_env.name);
            const configProcess = {
                name: environmentInfo.name,
                port: environmentInfo.port,
                pid: process[0].pm2_env.pm_id
            };
            await process_manager_1.ProcessListManager.addProcess(configProcess);
        }
        catch (error) {
            this.error(error.message);
        }
        finally {
            process_manager_1.ProcessManager.disconnect();
        }
    }
}
exports.default = Start;
Start.description = 'Start a mock API';
Start.examples = [
    '$ mockoon-cli start --data ~/export-data.json',
    '$ mockoon-cli start --data ~/export-data.json --index 0',
    '$ mockoon-cli start --data https://file-server/export-data.json --index 0',
    '$ mockoon-cli start --data ~/export-data.json --name "Mock environment"',
    '$ mockoon-cli start --data ~/export-data.json --name "Mock environment" --pname "proc1"'
];
Start.flags = Object.assign(Object.assign(Object.assign({}, command_constants_1.commonFlags), command_constants_1.startFlags), { pname: command_1.flags.string({
        char: 'N',
        description: 'Override the process name'
    }), 
    /**
     * /!\ Undocumented flag.
     * Mostly for internal use when `start `command is called during
     * a Docker image build.
     *
     * When using the `dockerize` command, file loading, validity checks,
     * migrations, etc. are all performed, and the single environment is
     * extracted in a separated file next to the generated Dockerfile.
     * It's easier to directly provide this file to the `start` command run
     * from the Dockerfile when building the Docker image rather than
     * having the image build failing due to a failure in the `start` command.
     */
    container: command_1.flags.boolean({
        char: 'c',
        hidden: true
    }) });
//# sourceMappingURL=start.js.map