"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.cleanDataFiles = exports.prepareData = exports.parseDataFile = void 0;
const commons_1 = require("@mockoon/commons");
const axios_1 = require("axios");
const fs_1 = require("fs");
const jsonfile_1 = require("jsonfile");
const mkdirp = require("mkdirp");
const path_1 = require("path");
const util_1 = require("util");
const config_1 = require("../config");
const messages_constants_1 = require("../constants/messages.constants");
const utils_1 = require("./utils");
/**
 * Load and parse a JSON data file
 *
 * @param filePath
 */
const parseDataFile = async (filePath) => {
    let dataExport;
    if (filePath.indexOf('http') !== 0) {
        dataExport = await jsonfile_1.readFile(filePath, 'utf-8');
    }
    else {
        const { data: responseData } = await axios_1.default.get(filePath, { timeout: 5000 });
        dataExport =
            typeof responseData === 'string'
                ? JSON.parse(responseData)
                : responseData;
    }
    // verify export file new format
    if (!dataExport.source || !dataExport.data) {
        throw new Error(messages_constants_1.Messages.CLI.DATA_FILE_TOO_OLD_ERROR);
    }
    // Extract all environments, eventually filter items of type 'route'
    const environments = dataExport.data.reduce((newEnvironments, dataItem) => {
        if (dataItem.type === 'environment') {
            newEnvironments.push(dataItem.item);
        }
        return newEnvironments;
    }, []);
    if (environments.length === 0) {
        throw new Error(messages_constants_1.Messages.CLI.ENVIRONMENT_NOT_AVAILABLE_ERROR);
    }
    return environments;
};
exports.parseDataFile = parseDataFile;
/**
 * Check if an environment can be run by the CLI and
 * migrate it if needed
 *
 * @param environment
 */
const migrateEnvironment = (environment) => {
    // environment data are too old: lastMigration is not present
    if (environment.lastMigration === undefined) {
        throw new Error(messages_constants_1.Messages.CLI.DATA_TOO_OLD_ERROR);
    }
    // environment data migrated with a more recent version (CLI does not include @mockoon/commons with required migrations)
    if (environment.lastMigration > commons_1.HighestMigrationId) {
        throw new Error(messages_constants_1.Messages.CLI.DATA_TOO_RECENT_ERROR);
    }
    // apply migrations
    commons_1.Migrations.forEach((migration) => {
        if (migration.id > environment.lastMigration) {
            migration.migrationFunction(environment);
        }
    });
    return environment;
};
/**
 * Check if data file is in the new format (with data and source)
 * and return the environment by index or name
 *
 * @param data
 * @param options
 */
const getEnvironment = (environments, options) => {
    let findError;
    // find environment by index
    if (options.index !== undefined && environments[options.index]) {
        return environments[options.index];
    }
    else {
        findError = util_1.format(messages_constants_1.Messages.CLI.ENVIRONMENT_NOT_FOUND_INDEX_ERROR, options.index);
    }
    // find by name
    if (options.name) {
        const foundEnvironment = environments.find((environment) => environment.name === options.name);
        if (foundEnvironment) {
            return foundEnvironment;
        }
        else {
            findError = util_1.format(messages_constants_1.Messages.CLI.ENVIRONMENT_NOT_FOUND_NAME_ERROR, options.name);
        }
    }
    throw new Error(findError);
};
/**
 * Load the data file, find and migrate the environment
 * copy the environment to a new temporary file.
 *
 * @param environments - path to the data file or export data
 * @param options
 */
const prepareData = async (environments, options, dockerfileDir) => {
    let environment = getEnvironment(environments, options);
    environment = migrateEnvironment(environment);
    // transform the provided name or env's name to be used as process name
    environment.name = utils_1.transformEnvironmentName(options.pname || environment.name);
    if (options.port !== undefined) {
        environment.port = options.port;
    }
    let dataFile = path_1.join(config_1.Config.dataPath, `${environment.name}.json`);
    // if we are building a Dockerfile, we want the data in the same folder
    if (dockerfileDir) {
        await mkdirp(dockerfileDir);
        dataFile = `${dockerfileDir}/${environment.name}.json`;
    }
    // save environment to data path
    await fs_1.promises.writeFile(dataFile, JSON.stringify(environment));
    return {
        name: environment.name,
        protocol: environment.https ? 'https' : 'http',
        port: environment.port,
        dataFile
    };
};
exports.prepareData = prepareData;
/**
 * Clean the temporary data files by deleting the ones with no
 * matching running process
 *
 * @param processes
 */
const cleanDataFiles = async (processes) => {
    const files = await fs_1.promises.readdir(config_1.Config.dataPath);
    files.forEach(async (file) => {
        if (processes.findIndex((process) => `${process.name}.json` === file) === -1) {
            await fs_1.promises.unlink(path_1.join(config_1.Config.dataPath, file));
        }
    });
};
exports.cleanDataFiles = cleanDataFiles;
//# sourceMappingURL=data.js.map